package video.pano.pano_rtc

import android.os.Handler
import android.os.Looper
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import kotlin.reflect.full.declaredMemberFunctions
import kotlin.reflect.jvm.javaMethod

abstract class FlutterWrapper : MethodChannel.MethodCallHandler, EventChannel.StreamHandler {

    protected lateinit var methodChannel: MethodChannel
    protected var eventChannel: EventChannel? = null
    private var eventSink: EventChannel.EventSink? = null
    private val handler = Handler(Looper.getMainLooper())

    protected fun emit(methodName: String, data: Map<String, Any?>?) {
        val event: MutableMap<String, Any?> = mutableMapOf("methodName" to methodName)
        data?.let { event.putAll(it) }

        handler.post{
            eventSink?.success(event)
        }
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        this::class.declaredMemberFunctions.find { it.name == call.method }?.let { function ->
            function.javaMethod?.let { method ->
                try {
                    val parameters = mutableListOf<Any?>()
                    call.arguments<Map<*, *>>()?.toMutableMap()?.let {
                        parameters.add(it)
                    }
                    method.invoke(this, *parameters.toTypedArray(), ResultCallback(result))
                    return@onMethodCall
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        }
        result.notImplemented()
    }

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        eventSink = events
    }

    override fun onCancel(arguments: Any?) {
        eventSink = null
    }

    open fun onDetachedFromEngine() {
        methodChannel.setMethodCallHandler(null)
    }
}