//
//  RtcAnnotationManager.swift
//  pano_rtc
//
//  Created by mark on 2020/12/30.
//

import Foundation
import PanoRtc

protocol RtcAnnotationManagerInterface {
    func getVideoAnnotation(_ params: NSDictionary, _ callback: Callback)
    
    func getShareAnnotation(_ params: NSDictionary, _ callback: Callback)
}

protocol RtcAnnotationManagerDelegate: AnyObject {
    func emit(_ methodName: String, _ data: Dictionary<String, Any?>?)
    
    func createVideoAnnotation(userId: UInt64, streamId: Int32, panoObj: PanoRtcAnnotation?) -> RtcAnnotation?
    
    func createShareAnnotation(userId: UInt64, panoObj: PanoRtcAnnotation?) -> RtcAnnotation?
}

@objc
class RtcAnnotationManager: NSObject, RtcAnnotationManagerInterface {
    weak var delegate: RtcAnnotationManagerDelegate?
    private var manager: PanoRtcAnnotationManager?
    private lazy var managerDelegate: PanoRtcAnnotationManagerDelegateHandler = {
        PanoRtcAnnotationManagerDelegateHandler() { [weak self] methodName, data in
            self?.delegate?.emit(methodName, data)
        }
    }()
    private(set) var videoAnnotationDic = Dictionary<String, RtcAnnotation>()
    private(set) var shareAnnotationDic = Dictionary<String, RtcAnnotation>()
    
    convenience init(manager: PanoRtcAnnotationManager?) {
        self.init()
        self.manager = manager
        self.manager?.setDelegate(self.managerDelegate)
    }
    
    func getVideoAnnotation(_ params: NSDictionary, _ callback: Callback) {
        let userId = params["userId"] as! UInt64
        let streamId = params["streamId"] as! Int32
        let videoId = "\(userId)-\(streamId)"
        let videoAnnotation = videoAnnotationDic[videoId]
        if videoAnnotation == nil {
            let annotion =
                delegate?.createVideoAnnotation(
                    userId: userId,
                    streamId: streamId,
                    panoObj: manager?.videoAnnotation(userId, stream: streamId))
            if annotion == nil {
                callback.code(PanoResult.invalidState)
                return
            }
            videoAnnotationDic[videoId] = annotion
        }
        callback.code(PanoResult.OK)
    }
    
    func getShareAnnotation(_ params: NSDictionary, _ callback: Callback) {
        let userId = params["userId"] as! UInt64
        let videoId = "\(userId)"
        let shareAnnotation = shareAnnotationDic[videoId]
        if shareAnnotation == nil {
            let annotion =
                delegate?.createShareAnnotation(
                    userId: userId,
                    panoObj: manager?.shareAnnotation(userId))
            if annotion == nil {
                callback.code(PanoResult.invalidState)
                return
            }
            shareAnnotationDic[videoId] = annotion
        }
        callback.code(PanoResult.OK)
    }
}
