//
//  RtcWhiteboard.swift
//  pano_rtc
//
//  Created by mark on 2020/12/28.
//

import Foundation
import PanoRtc

protocol RtcWhiteboardInterfce {
    func open(_ params: NSDictionary, _ callback: Callback)
    
    func close(_ callback: Callback)
    
    func stop(_ callback: Callback)
    
    func setRoleType(_ params: NSDictionary, _ callback: Callback)
    
    func setToolType(_ params: NSDictionary, _ callback: Callback)
    
    func setLineWidth(_ params: NSDictionary, _ callback: Callback)
    
    func setFillType(_ params: NSDictionary, _ callback: Callback)
    
    func setFillColor(_ params: NSDictionary, _ callback: Callback)
    
    func setForegroundColor(_ params: NSDictionary, _ callback: Callback)
    
    func setBackgroundColor(_ params: NSDictionary, _ callback: Callback)
    
    func setFontStyle(_ params: NSDictionary, _ callback: Callback)
    
    func setFontSize(_ params: NSDictionary, _ callback: Callback)
    
    func setBackgroundImageScalingMode(_ params: NSDictionary, _ callback: Callback)
    
    func setBackgroundImage(_ params: NSDictionary, _ callback: Callback)
    
    func setBackgroundImageWithPage(_ params: NSDictionary, _ callback: Callback)

    func getCurrentPageNumber(_ callback: Callback)
    
    func getTotalNumberOfPages(_ callback: Callback)
    
    func addPage(_ params: NSDictionary, _ callback: Callback)
    
    func insertPage(_ params: NSDictionary, _ callback: Callback)
    
    func removePage(_ params: NSDictionary, _ callback: Callback)
    
    func gotoPage(_ params: NSDictionary, _ callback: Callback)
    
    func nextPage(_ callback: Callback)
    
    func prevPage(_ callback: Callback)
    
    func addBackgroundImages(_ params: NSDictionary, _ callback: Callback)
    
    func createDocWithImages(_ params: NSDictionary, _ callback: Callback)
    
    func createDocWithFilePath(_ params: NSDictionary, _ callback: Callback)
    
    func deleteDoc(_ params: NSDictionary, _ callback: Callback)
    
    func switchDoc(_ params: NSDictionary, _ callback: Callback)
    
    func saveDocToImages(_ params: NSDictionary, _ callback: Callback)

    func enumerateFiles(_ callback: Callback)
    
    func getCurrentFileId(_ callback: Callback)
    
    func clearContents(_ params: NSDictionary, _ callback: Callback)
    
    func clearUserContents(_ params: NSDictionary, _ callback: Callback)
    
    func undo(_ callback: Callback)
    
    func redo(_ callback: Callback)
    
    func getCurrentScaleFactor(_ callback: Callback)
    
    func setCurrentScaleFactor(_ params: NSDictionary, _ callback: Callback)
    
    func snapshot(_ params: NSDictionary, _ callback: Callback)
    
    func sendMessage(_ params: NSDictionary, _ callback: Callback)
    
    func broadcastMessage(_ params: NSDictionary, _ callback: Callback)
    
    func setOption(_ params: NSDictionary, _ callback: Callback)
}

protocol RtcWhiteboardDelegate: AnyObject {
    func emit(_ methodName: String, _ data: Dictionary<String, Any?>?)
}

@objc
class RtcWhiteboard: NSObject, RtcWhiteboardInterfce {
    weak var delegate: RtcWhiteboardDelegate?
    public var whiteboardId: String?
    private var engine: PanoRtcWhiteboard?
    private lazy var engineDelegate: PanoRtcWhiteboardDelegateHandler = {
        PanoRtcWhiteboardDelegateHandler() { [weak self] methodName, data in
            self?.delegate?.emit(methodName, data)
        }
    }()
    
    convenience init(id: String, whiteboardEngine: PanoRtcWhiteboard?) {
        self.init()
        self.whiteboardId = id
        self.engine = whiteboardEngine
        self.engine?.setDelegate(self.engineDelegate)
    }
    
    func open(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.open(params["view"] as! UIView))
    }
    
    func close(_ callback: Callback) {
        callback.code(engine?.close())
    }
    
    func stop(_ callback: Callback) {
        callback.code(engine?.stop())
    }
    
    func setRoleType(_ params: NSDictionary, _ callback: Callback) {
        callback.code(
            engine?.setRoleType(PanoWBRoleType(rawValue: params["type"] as! Int)!))
    }
    
    func setToolType(_ params: NSDictionary, _ callback: Callback) {
        callback.code(
            engine?.setToolType(PanoWBToolType(rawValue: params["type"] as! Int)!))
    }
    
    func setLineWidth(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.setLineWidth(params["width"] as! UInt32))
    }
    
    func setFillType(_ params: NSDictionary, _ callback: Callback) {
        callback.code(
            engine?.setFillType(PanoWBFillType(rawValue: params["type"] as! Int)!))
    }
    
    func setFillColor(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.setFill(PanoWBColor(map: params["color"] as! Dictionary)))
    }
    
    func setForegroundColor(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.setForegroundColor(PanoWBColor(map: params["color"] as! Dictionary)))
    }
    
    func setBackgroundColor(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.setBackgroundColor(PanoWBColor(map: params["color"] as! Dictionary)))
    }
    
    func setFontStyle(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.setFontStyle(PanoWBFontStyle(rawValue: params["style"] as! Int)!))
    }
    
    func setFontSize(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.setFontSize(params["size"] as! UInt32))
    }
    
    func setBackgroundImageScalingMode(_ params: NSDictionary, _ callback: Callback) {
        callback.code(
            engine?.setBackgroundImageScalingMode(
                PanoWBImageScalingMode(rawValue: params["mode"] as! Int)!))
    }
    
    func setBackgroundImage(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.setBackgroundImage(params["imageUrl"] as! String))
    }
    
    func setBackgroundImageWithPage(_ params: NSDictionary, _ callback: Callback) {
        callback.code(
            engine?.setBackgroundImage(params["imageUrl"] as! String,
                                       withPageNumber: params["pageNo"] as! PanoWBPageNumber))
    }
    
    func getCurrentPageNumber(_ callback: Callback) {
        callback.resolve(engine) { $0.getCurrentPageNumber() }
    }
    
    func getTotalNumberOfPages(_ callback: Callback) {
        callback.resolve(engine) { $0.getTotalNumberOfPages() }
    }
    
    func addPage(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.addPage(params["autoSwitch"] as! Bool))
    }
    
    func insertPage(_ params: NSDictionary, _ callback: Callback) {
        callback.code(
            engine?.insertPage(params["pageNo"] as! PanoWBPageNumber,
                               autoSwitch: params["autoSwitch"] as! Bool))
    }
    
    func removePage(_ params: NSDictionary, _ callback: Callback) {
        callback.code(
            engine?.removePage(params["pageNo"] as! PanoWBPageNumber,
                               switchNext: params["switchNext"] as! Bool))
    }
    
    func gotoPage(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.gotoPage(params["pageNo"] as! PanoWBPageNumber))
    }
    
    func nextPage(_ callback: Callback) {
        callback.code(engine?.nextPage())
    }
    
    func prevPage(_ callback: Callback) {
        callback.code(engine?.prevPage())
    }
    
    func addBackgroundImages(_ params: NSDictionary, _ callback: Callback) {
        callback.resolve(engine) { $0.addBackgroundImages(params["urls"] as! [String]) }
    }
    
    func createDocWithImages(_ params: NSDictionary, _ callback: Callback) {
        callback.resolve(engine) { $0.createDoc(withImages: params["urls"] as! [String]) }
    }
    
    func createDocWithFilePath(_ params: NSDictionary, _ callback: Callback) {
        callback.resolve(engine) { $0.createDoc(withFilePath: params["filePath"] as! String) }
    }
    
    func deleteDoc(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.deleteDoc(params["fileId"] as! String))
    }
    
    func switchDoc(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.switchDoc(params["fileId"] as! String))
    }
    
    func saveDocToImages(_ params: NSDictionary, _ callback: Callback) {
        callback.code(
            engine?.saveDoc(toImages: params["fileId"] as! String,
                            path: params["outputDir"] as! String))
    }
    
    func enumerateFiles(_ callback: Callback) {
        callback.resolve(engine) { $0.enumerateFiles() }
    }
    
    func getCurrentFileId(_ callback: Callback) {
        callback.resolve(engine) { $0.getCurrentFileId() }
    }
    
    func clearContents(_ params: NSDictionary, _ callback: Callback) {
        callback.code(
            engine?.clearContents(params["curPage"] as! Bool,
                                  with: PanoWbClearType(rawValue: params["type"] as! Int)!))
    }
    
    func clearUserContents(_ params: NSDictionary, _ callback: Callback) {
        callback.code(
            engine?.clearUserContents(params["userId"] as! UInt64,
                                      currentPage: params["curPage"] as! Bool,
                                      with: PanoWbClearType(rawValue: params["type"] as! Int)!))
    }
    
    func undo(_ callback: Callback) {
        callback.code(engine?.undo())
    }
    
    func redo(_ callback: Callback) {
        callback.code(engine?.redo())
    }
    
    func getCurrentScaleFactor(_ callback: Callback) {
        callback.resolve(engine) { $0.getCurrentScaleFactor() }
    }
    
    func setCurrentScaleFactor(_ params: NSDictionary, _ callback: Callback) {
        callback.code(engine?.setCurrentScaleFactor(params["scale"] as! Float32))
    }
    
    func snapshot(_ params: NSDictionary, _ callback: Callback) {
        callback.code(
            engine?.snapshot(PanoWBSnapshotMode(rawValue: params["mode"] as! Int)!,
                             path: params["outputDir"] as! String))
    }
    
    func sendMessage(_ params: NSDictionary, _ callback: Callback) {
        guard let data = params["message"] as? FlutterStandardTypedData else {
            callback.code(PanoResult.invalidArgs)
            return
        }
        
        callback.code(engine?.sendMessage(data.data, toUser: params["userId"] as! UInt64))
    }
    
    func broadcastMessage(_ params: NSDictionary, _ callback: Callback) {
        guard let data = params["message"] as? FlutterStandardTypedData else {
            callback.code(PanoResult.invalidArgs)
            return
        }
        
        callback.code(engine?.broadcastMessage(data.data))
    }
    
    func setOption(_ params: NSDictionary, _ callback: Callback) {
        var option: NSObject? = nil
        let type = PanoWBOptionType(rawValue: params["type"] as! Int)
        var isValid = true
        switch type {
        case .fileCachePath:
            option = params["option"] as! NSString
        case .enableUIResponse:
            option = NSNumber(value: params["option"] as! Bool)
        default:
            isValid = false
        }
        if isValid {
            callback.code(engine?.setOption(option, for: type!))
        } else {
            callback.code(PanoResult.invalidArgs)
        }
    }
}
